# NBD client library in userspace
# WARNING: THIS FILE IS GENERATED FROM
# generator/generator
# ANY CHANGES YOU MAKE TO THIS FILE WILL BE LOST.
#
# Copyright (C) 2013-2019 Red Hat Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

'''
Python bindings for libnbd

import nbd
h = nbd.NBD ()
h.connect_tcp ("localhost", "nbd")
buf = h.pread (512, 0)

Read the libnbd(3) man page to find out how to use the API.
'''

import libnbdmod

AIO_DIRECTION_READ             = 1
AIO_DIRECTION_WRITE            = 2
AIO_DIRECTION_BOTH             = 3
CMD_FLAG_FUA                   = 1
CMD_FLAG_NO_HOLE               = 2
CMD_FLAG_REQ_ONE               = 8

# AIO buffer functions.
def aio_buffer (len):
    '''allocate an AIO buffer used for nbd.aio_pread and nbd.aio_pwrite'''
    return libnbdmod.alloc_aio_buffer (len)

def aio_buffer_from_bytearray (ba):
    '''create an AIO buffer from a bytearray'''
    return libnbdmod.aio_buffer_from_bytearray (ba)
def aio_buffer_to_bytearray (buf):
    '''copy an AIO buffer into a bytearray'''
    return libnbdmod.aio_buffer_to_bytearray (buf)

class NBD (object):
    '''NBD handle'''

    def __init__ (self):
        '''create a new NBD handle'''
        self._o = libnbdmod.create ()

    def __del__ (self):
        '''close the NBD handle and underlying connection'''
        libnbdmod.close (self._o)

    def set_debug (self, debug):
        '''▶ set or clear the debug flag

    Set or clear the debug flag. When debugging is enabled,
    debugging messages from the library are printed to
    stderr, unless a debugging callback has been defined too
    (see "nbd.set_debug_callback") in which case they are
    sent to that function. This flag defaults to false on
    newly created handles, except if "DEBUG=1" is set in the
    environment in which case it defaults to true.
'''
        return libnbdmod.set_debug (self._o, debug)

    def get_debug (self):
        '''▶ return the state of the debug flag

    Return the state of the debug flag on this handle.
'''
        return libnbdmod.get_debug (self._o)

    def set_debug_callback (self, data, debug_fn):
        '''▶ set the debug callback

    Set the debug callback. This function is called when the
    library emits debug messages, when debugging is enabled
    on a handle. The callback parameters are "data" passed
    to this function, the name of the libnbd function
    emitting the debug message ("context"), and the message
    itself ("msg"). If no debug callback is set on a handle
    then messages are printed on "stderr".

    The callback should not call "nbd.*" APIs on the same
    handle since it can be called while holding the handle
    lock and will cause a deadlock.
'''
        return libnbdmod.set_debug_callback (self._o, data, debug_fn)

    def set_export_name (self, export_name):
        '''▶ set the export name

    For servers which require an export name or can serve
    different content on different exports, set the
    "export_name" to connect to. This is only relevant for
    the newstyle protocol. This call may be skipped if using
    "nbd.connect_uri" to connect to a URI that includes an
    export name. The default is to use the empty string.
'''
        return libnbdmod.set_export_name (self._o, export_name)

    def get_export_name (self):
        '''▶ get the export name

    Get the export name associated with the handle.
'''
        return libnbdmod.get_export_name (self._o)

    def set_tls (self, tls):
        '''▶ enable or require TLS (authentication and encryption)

    Enable or require TLS (authenticated and encrypted
    connections) to the NBD server. The possible settings
    are:

    "tls=0"
        Disable TLS. (The default setting, unless using
        "nbd.connect_uri" with a URI that requires TLS)

    "tls=1"
        Enable TLS if possible. In some cases this will fall
        back to an unencrypted and/or unauthenticated
        connection if TLS could not be established. However
        some servers will drop the connection if TLS fails
        so fallback may not be possible.

    "tls=2"
        Require an encrypted and authenticated TLS
        connection. Always fail to connect if the connection
        is not encrypted and authenticated.

    As well as calling this you may also need to supply the
    path to the certificates directory
    ("nbd.set_tls_certificates"), the username
    ("nbd.set_tls_username") and/or the Pre-Shared Keys
    (PSK) file ("nbd.set_tls_psk_file"). For now, when using
    "nbd.connect_uri", any URI query parameters related to
    TLS are not handled automatically. Setting the level
    higher than zero will fail if libnbd was not compiled
    against gnutls; you can test whether this is the case
    with "nbd.supports_tls".

    For more information see "ENCRYPTION AND AUTHENTICATION"
    in libnbd(3).
'''
        return libnbdmod.set_tls (self._o, tls)

    def get_tls (self):
        '''▶ get the current TLS setting

    Get the current TLS setting. See "nbd.set_tls".
'''
        return libnbdmod.get_tls (self._o)

    def set_tls_certificates (self, dir):
        '''▶ set the path to the TLS certificates directory

    Set the path to the TLS certificates directory. If not
    set and TLS is used then a compiled in default is used.
    For root this is "/etc/pki/libnbd/". For non-root this
    is "$HOME/.pki/libnbd" and "$HOME/.config/pki/libnbd".
    If none of these directories can be found then the
    system trusted CAs are used.

    This function may be called regardless of whether TLS is
    supported, but will have no effect unless "nbd.set_tls"
    is also used to request or require TLS.
'''
        return libnbdmod.set_tls_certificates (self._o, dir)

    def set_tls_verify_peer (self, verify):
        '''▶ set whether we verify the identity of the server

    Set this flag to control whether libnbd will verify the
    identity of the server from the server's certificate and
    the certificate authority. This defaults to true when
    connecting to TCP servers using TLS certificate
    authentication, and false otherwise.

    This function may be called regardless of whether TLS is
    supported, but will have no effect unless "nbd.set_tls"
    is also used to request or require TLS.
'''
        return libnbdmod.set_tls_verify_peer (self._o, verify)

    def get_tls_verify_peer (self):
        '''▶ get whether we verify the identity of the server

    Get the verify peer flag.
'''
        return libnbdmod.get_tls_verify_peer (self._o)

    def set_tls_username (self, username):
        '''▶ set the TLS username

    Set the TLS client username. This is used if
    authenticating with PSK over TLS is enabled. If not set
    then the local username is used.

    This function may be called regardless of whether TLS is
    supported, but will have no effect unless "nbd.set_tls"
    is also used to request or require TLS.
'''
        return libnbdmod.set_tls_username (self._o, username)

    def get_tls_username (self):
        '''▶ get the current TLS username

    Get the current TLS username. See
    "nbd.set_tls_username".
'''
        return libnbdmod.get_tls_username (self._o)

    def set_tls_psk_file (self, filename):
        '''▶ set the TLS Pre-Shared Keys (PSK) filename

    Set the TLS Pre-Shared Keys (PSK) filename. This is used
    if trying to authenticate to the server using with a
    pre-shared key. There is no default so if this is not
    set then PSK authentication cannot be used to connect to
    the server.

    This function may be called regardless of whether TLS is
    supported, but will have no effect unless "nbd.set_tls"
    is also used to request or require TLS.
'''
        return libnbdmod.set_tls_psk_file (self._o, filename)

    def add_meta_context (self, name):
        '''▶ ask server to negotiate metadata context

    During connection libnbd can negotiate zero or more
    metadata contexts with the server. Metadata contexts are
    features (such as "base:allocation") which describe
    information returned by the "nbd.block_status" command
    (for "base:allocation" this is whether blocks of data
    are allocated, zero or sparse).

    This call adds one metadata context to the list to be
    negotiated. You can call it as many times as needed. The
    list is initially empty when the handle is created.

    Not all servers support all metadata contexts. To learn
    if a context was actually negotiated, call
    "nbd.can_meta_context" after connecting.

    The single parameter is the name of the metadata
    context, for example "base:allocation".

    Other metadata contexts are server-specific, but include
    "qemu:dirty-bitmap:..." for qemu-nbd (see qemu-nbd *-B*
    option). See also "nbd.block_status".
'''
        return libnbdmod.add_meta_context (self._o, name)

    def connect_uri (self, uri):
        '''▶ connect to NBD URI

    Connect (synchronously) to an NBD server and export by
    specifying the NBD URI. This call parses the URI and may
    call "nbd.set_export_name" and "nbd.set_tls" as needed,
    followed by "nbd.connect_tcp" or "nbd.connect_unix".
    This call returns when the connection has been made.

    This call will fail if libnbd was not compiled with
    libxml2; you can test whether this is the case with
    "nbd.supports_uri". Support for URIs that require TLS
    will fail if libnbd was not compiled with gnutls; you
    can test whether this is the case with
    "nbd.supports_tls".
'''
        return libnbdmod.connect_uri (self._o, uri)

    def connect_unix (self, unixsocket):
        '''▶ connect to NBD server over a Unix domain socket

    Connect (synchronously) over the named Unix domain
    socket ("unixsocket") to an NBD server running on the
    same machine. This call returns when the connection has
    been made.
'''
        return libnbdmod.connect_unix (self._o, unixsocket)

    def connect_tcp (self, hostname, port):
        '''▶ connect to NBD server over a TCP port

    Connect (synchronously) to the NBD server listening on
    "hostname:port". The "port" may be a port name such as
    "nbd", or it may be a port number as a string such as
    "10809". This call returns when the connection has been
    made.
'''
        return libnbdmod.connect_tcp (self._o, hostname, port)

    def connect_command (self, argv):
        '''▶ connect to NBD server command

    Run the command as a subprocess and connect to it over
    stdin/stdout. This is for use with NBD servers which can
    behave like inetd clients, such as "nbdkit --single".
'''
        return libnbdmod.connect_command (self._o, argv)

    def read_only (self):
        '''▶ is the NBD export read-only?

    Returns true if the NBD export is read-only; writes and
    write-like operations will fail.
'''
        return libnbdmod.read_only (self._o)

    def can_flush (self):
        '''▶ does the server support the flush command?

    Returns true if the server supports the flush command
    (see "nbd.flush", "nbd.aio_flush"). Returns false if the
    server does not.
'''
        return libnbdmod.can_flush (self._o)

    def can_fua (self):
        '''▶ does the server support the FUA flag?

    Returns true if the server supports the FUA flag on
    certain commands (see "nbd.pwrite").
'''
        return libnbdmod.can_fua (self._o)

    def is_rotational (self):
        '''▶ is the NBD disk rotational (like a disk)?

    Returns true if the disk exposed over NBD is rotational
    (like a traditional floppy or hard disk). Returns false
    if the disk has no penalty for random access (like an
    SSD or RAM disk).
'''
        return libnbdmod.is_rotational (self._o)

    def can_trim (self):
        '''▶ does the server support the trim command?

    Returns true if the server supports the trim command
    (see "nbd.trim", "nbd.aio_trim"). Returns false if the
    server does not.
'''
        return libnbdmod.can_trim (self._o)

    def can_zero (self):
        '''▶ does the server support the zero command?

    Returns true if the server supports the zero command
    (see "nbd.zero", "nbd.aio_zero"). Returns false if the
    server does not.
'''
        return libnbdmod.can_zero (self._o)

    def can_multi_conn (self):
        '''▶ does the server support multi-conn?

    Returns true if the server supports multi-conn. Returns
    false if the server does not.

    It is not safe to open multiple handles connecting to
    the same server if you will write to the server and the
    server does not advertize multi-conn support. The safe
    way to check for this is to open one connection, check
    this flag is true, then open further connections as
    required.
'''
        return libnbdmod.can_multi_conn (self._o)

    def can_cache (self):
        '''▶ does the server support the cache command?

    Returns true if the server supports the cache command
    (see "nbd.cache", "nbd.aio_cache"). Returns false if the
    server does not.
'''
        return libnbdmod.can_cache (self._o)

    def can_meta_context (self, metacontext):
        '''▶ does the server support a specific meta context?

    Returns true if the server supports the given meta
    context (see "nbd.add_meta_context"). Returns false if
    the server does not.

    The single parameter is the name of the metadata
    context, for example "base:allocation".
'''
        return libnbdmod.can_meta_context (self._o, metacontext)

    def get_size (self):
        '''▶ return the export size

    Returns the size in bytes of the NBD export.
'''
        return libnbdmod.get_size (self._o)

    def pread (self, count, offset, flags=0):
        '''▶ read from the NBD server

    Issue a read command to the NBD server for the range
    starting at "offset" and ending at "offset" + "count" -
    1. NBD can only read all or nothing using this call. The
    call returns when the data has been read fully into
    "buf" or there is an error.

    The "flags" parameter must be 0 for now (it exists for
    future NBD protocol extensions).
'''
        return libnbdmod.pread (self._o, count, offset, flags)

    def pwrite (self, buf, offset, flags=0):
        '''▶ write to the NBD server

    Issue a write command to the NBD server, writing the
    data in "buf" to the range starting at "offset" and
    ending at "offset" + "count" - 1. NBD can only write all
    or nothing using this call. The call returns when the
    command has been acknowledged by the server, or there is
    an error.

    The "flags" parameter may be 0 for no flags, or may
    contain "CMD_FLAG_FUA" meaning that the server should
    not return until the data has been committed to
    permanent storage (if that is supported - some servers
    cannot do this, see "nbd.can_fua").
'''
        return libnbdmod.pwrite (self._o, buf, offset, flags)

    def shutdown (self):
        '''▶ disconnect from the NBD server

    Issue the disconnect command to the NBD server. This is
    a nice way to tell the server we are going away, but
    from the client's point of view has no advantage over
    abruptly closing the connection (see "nbd.close").

    This function works whether or not the handle is ready
    for transmission of commands, and as such does not take
    a "flags" parameter. If more fine-grained control is
    needed, see "nbd.aio_disconnect".
'''
        return libnbdmod.shutdown (self._o)

    def flush (self, flags=0):
        '''▶ send flush command to the NBD server

    Issue the flush command to the NBD server. The function
    should return when all write commands which have
    completed have been committed to permanent storage on
    the server. Note this will return an error if
    "nbd.can_flush" is false.

    The "flags" parameter must be 0 for now (it exists for
    future NBD protocol extensions).
'''
        return libnbdmod.flush (self._o, flags)

    def trim (self, count, offset, flags=0):
        '''▶ send trim command to the NBD server

    Issue a trim command to the NBD server, which if
    supported by the server causes a hole to be punched in
    the backing store starting at "offset" and ending at
    "offset" + "count" - 1. The call returns when the
    command has been acknowledged by the server, or there is
    an error.

    The "flags" parameter may be 0 for no flags, or may
    contain "CMD_FLAG_FUA" meaning that the server should
    not return until the data has been committed to
    permanent storage (if that is supported - some servers
    cannot do this, see "nbd.can_fua").
'''
        return libnbdmod.trim (self._o, count, offset, flags)

    def cache (self, count, offset, flags=0):
        '''▶ send cache (prefetch) command to the NBD server

    Issue the cache (prefetch) command to the NBD server,
    which if supported by the server causes data to be
    prefetched into faster storage by the server, speeding
    up a subsequent "nbd.pread" call. The server can also
    silently ignore this command. Note this will return an
    error if "nbd.can_cache" is false.

    The "flags" parameter must be 0 for now (it exists for
    future NBD protocol extensions).
'''
        return libnbdmod.cache (self._o, count, offset, flags)

    def zero (self, count, offset, flags=0):
        '''▶ send write zeroes command to the NBD server

    Issue a write zeroes command to the NBD server, which if
    supported by the server causes a zeroes to be written
    efficiently starting at "offset" and ending at "offset"
    + "count" - 1. The call returns when the command has
    been acknowledged by the server, or there is an error.

    The "flags" parameter may be 0 for no flags, or may
    contain "CMD_FLAG_FUA" meaning that the server should
    not return until the data has been committed to
    permanent storage (if that is supported - some servers
    cannot do this, see "nbd.can_fua"), and/or
    "CMD_FLAG_NO_HOLE" meaning that the server should favor
    writing actual allocated zeroes over punching a hole.
'''
        return libnbdmod.zero (self._o, count, offset, flags)

    def block_status (self, count, offset, data, extent, flags=0):
        '''▶ send block status command to the NBD server

    Issue the block status command to the NBD server. If
    supported by the server, this causes metadata context
    information about blocks beginning from the specified
    offset to be returned. The "count" parameter is a hint:
    the server may choose to return less status, or the
    final block may extend beyond the requested range. If
    multiple contexts are supported, the number of blocks
    and cumulative length of those blocks need not be
    identical between contexts.

    Depending on which metadata contexts were enabled before
    connecting (see "nbd.add_meta_context") and which are
    supported by the server (see "nbd.can_meta_context")
    this call returns information about extents by calling
    back to the extent function. The callback cannot call
    "nbd.*" APIs on the same handle since it holds the
    handle lock and will cause a deadlock. If the callback
    returns -1, any remaining contexts will be ignored and
    the overall block status command will fail with the same
    value of "errno" left after the failing callback.

    The extent function is called once per type of metadata
    available. The "metacontext" parameter is a string such
    as "base:allocation". The "entries" array is an array of
    pairs of integers with the first entry in each pair
    being the length (in bytes) of the block and the second
    entry being a status/flags field which is specific to
    the metadata context. (The number of pairs passed to the
    function is "nr_entries/2".) The NBD protocol document
    in the section about "NBD_REPLY_TYPE_BLOCK_STATUS"
    describes the meaning of this array.

    It is possible for the extent function to be called more
    times than you expect (if the server is buggy), so
    always check the "metacontext" field to ensure you are
    receiving the data you expect. It is also possible that
    the extent function is not called at all, even for
    metadata contexts that you requested. This indicates
    either that the server doesn't support the context or
    for some other reason cannot return the data.

    The "flags" parameter may be 0 for no flags, or may
    contain "CMD_FLAG_REQ_ONE" meaning that the server
    should return only one extent per metadata context where
    that extent does not exceed "count" bytes; however,
    libnbd does not validate that the server obeyed the
    flag.
'''
        return libnbdmod.block_status (self._o, count, offset, data, extent, flags)

    def poll (self, timeout):
        '''▶ poll the handle once

    This is a simple implementation of poll(2) which is used
    internally by synchronous API calls. It is mainly useful
    as an example of how you might integrate libnbd with
    your own main loop, rather than being intended as
    something you would use.
'''
        return libnbdmod.poll (self._o, timeout)

    def aio_connect (self, addr):
        '''▶ connect to the NBD server

    Begin connecting to the NBD server. Parameters behave as
    documented in "nbd.connect".

    You can check if the connection is still connecting by
    calling "nbd.aio_is_connecting", or if it has connected
    to the server and completed the NBD handshake by calling
    "nbd.aio_is_ready", on the connection.
'''
        return libnbdmod.aio_connect (self._o, addr)

    def aio_connect_uri (self, uri):
        '''▶ connect to an NBD URI

    Begin connecting to the NBD URI "uri". Parameters behave
    as documented in "nbd.connect_uri".

    You can check if the connection is still connecting by
    calling "nbd.aio_is_connecting", or if it has connected
    to the server and completed the NBD handshake by calling
    "nbd.aio_is_ready", on the connection.

    This call will fail if libnbd was not compiled with
    libxml2; you can test whether this is the case with
    "nbd.supports_uri". Support for URIs that require TLS
    will fail if libnbd was not compiled with gnutls; you
    can test whether this is the case with
    "nbd.supports_tls".
'''
        return libnbdmod.aio_connect_uri (self._o, uri)

    def aio_connect_unix (self, unixsocket):
        '''▶ connect to the NBD server over a Unix domain socket

    Begin connecting to the NBD server over Unix domain
    socket ("unixsocket"). Parameters behave as documented
    in "nbd.connect_unix".

    You can check if the connection is still connecting by
    calling "nbd.aio_is_connecting", or if it has connected
    to the server and completed the NBD handshake by calling
    "nbd.aio_is_ready", on the connection.
'''
        return libnbdmod.aio_connect_unix (self._o, unixsocket)

    def aio_connect_tcp (self, hostname, port):
        '''▶ connect to the NBD server over a TCP port

    Begin connecting to the NBD server listening on
    "hostname:port". Parameters behave as documented in
    "nbd.connect_tcp".

    You can check if the connection is still connecting by
    calling "nbd.aio_is_connecting", or if it has connected
    to the server and completed the NBD handshake by calling
    "nbd.aio_is_ready", on the connection.
'''
        return libnbdmod.aio_connect_tcp (self._o, hostname, port)

    def aio_connect_command (self, argv):
        '''▶ connect to the NBD server

    Run the command as a subprocess and begin connecting to
    it over stdin/stdout. Parameters behave as documented in
    "nbd.connect_command".

    You can check if the connection is still connecting by
    calling "nbd.aio_is_connecting", or if it has connected
    to the server and completed the NBD handshake by calling
    "nbd.aio_is_ready", on the connection.
'''
        return libnbdmod.aio_connect_command (self._o, argv)

    def aio_pread (self, buf, offset, flags=0):
        '''▶ read from the NBD server

    Issue a read command to the NBD server. This returns the
    unique positive 64 bit handle for this command, or -1 on
    error. To check if the command completed, call
    "nbd.aio_command_completed". Note that you must ensure
    "buf" is valid until the command has completed. Other
    parameters behave as documented in "nbd.pread".
'''
        return libnbdmod.aio_pread (self._o, buf, offset, flags)

    def aio_pwrite (self, buf, offset, flags=0):
        '''▶ write to the NBD server

    Issue a write command to the NBD server. This returns
    the unique positive 64 bit handle for this command, or
    -1 on error. To check if the command completed, call
    "nbd.aio_command_completed". Note that you must ensure
    "buf" is valid until the command has completed. Other
    parameters behave as documented in "nbd.pwrite".
'''
        return libnbdmod.aio_pwrite (self._o, buf, offset, flags)

    def aio_disconnect (self, flags=0):
        '''▶ disconnect from the NBD server

    Issue the disconnect command to the NBD server. This is
    not a normal command because NBD servers are not obliged
    to send a reply. Instead you should wait for
    "nbd.aio_is_closed" to become true on the connection.

    The "flags" parameter must be 0 for now (it exists for
    future NBD protocol extensions). There is no direct
    synchronous counterpart; however, "nbd.shutdown" will
    call this function if appropriate.
'''
        return libnbdmod.aio_disconnect (self._o, flags)

    def aio_flush (self, flags=0):
        '''▶ send flush command to the NBD server

    Issue the flush command to the NBD server. This returns
    the unique positive 64 bit handle for this command, or
    -1 on error. To check if the command completed, call
    "nbd.aio_command_completed". Parameters behave as
    documented in "nbd.flush".
'''
        return libnbdmod.aio_flush (self._o, flags)

    def aio_trim (self, count, offset, flags=0):
        '''▶ send trim command to the NBD server

    Issue a trim command to the NBD server. This returns the
    unique positive 64 bit handle for this command, or -1 on
    error. To check if the command completed, call
    "nbd.aio_command_completed". Parameters behave as
    documented in "nbd.trim".
'''
        return libnbdmod.aio_trim (self._o, count, offset, flags)

    def aio_cache (self, count, offset, flags=0):
        '''▶ send cache (prefetch) command to the NBD server

    Issue the cache (prefetch) command to the NBD server.
    This returns the unique positive 64 bit handle for this
    command, or -1 on error. To check if the command
    completed, call "nbd.aio_command_completed". Parameters
    behave as documented in "nbd.cache".
'''
        return libnbdmod.aio_cache (self._o, count, offset, flags)

    def aio_zero (self, count, offset, flags=0):
        '''▶ send write zeroes command to the NBD server

    Issue a write zeroes command to the NBD server. This
    returns the unique positive 64 bit handle for this
    command, or -1 on error. To check if the command
    completed, call "nbd.aio_command_completed". Parameters
    behave as documented in "nbd.zero".
'''
        return libnbdmod.aio_zero (self._o, count, offset, flags)

    def aio_block_status (self, count, offset, data, extent, flags=0):
        '''▶ send block status command to the NBD server

    Send the block status command to the NBD server. This
    returns the unique positive 64 bit handle for this
    command, or -1 on error. To check if the command
    completed, call "nbd.aio_command_completed". Parameters
    behave as documented in "nbd.block_status".
'''
        return libnbdmod.aio_block_status (self._o, count, offset, data, extent, flags)

    def aio_get_fd (self):
        '''▶ return file descriptor associated with this connection

    Return the underlying file descriptor associated with
    this connection. You can use this to check if the file
    descriptor is ready for reading or writing and call
    "nbd.aio_notify_read" or "nbd.aio_notify_write". See
    also "nbd.aio_get_direction". Do not do anything else
    with the file descriptor.
'''
        return libnbdmod.aio_get_fd (self._o)

    def aio_get_direction (self):
        '''▶ return the read or write direction

    Return the current direction of this connection, which
    means whether we are next expecting to read data from
    the server, write data to the server, or both. It
    returns

    "AIO_DIRECTION_READ" = 1
        We are expected next to read from the server. If
        using poll(2) you would set "events = POLLIN". If
        "revents" returns "POLLIN" you would then call
        "nbd.aio_notify_read".

    "AIO_DIRECTION_WRITE" = 2
        We are expected next to write to the server. If
        using poll(2) you would set "events = POLLOUT". If
        "revents" returns "POLLOUT" you would then call
        "nbd.aio_notify_write".

    "AIO_DIRECTION_BOTH" = 3
        We are expected next to either read or write to the
        server. If using poll(2) you would set "events =
        POLLIN|POLLOUT". If one of "POLLIN" or "POLLOUT" is
        returned, then see above. However note that you
        shouldn't call "nbd.aio_notify_read" and
        "nbd.aio_notify_write" because calling the first one
        will change the state of the connection, possibly
        invalidating the second notification.
'''
        return libnbdmod.aio_get_direction (self._o)

    def aio_notify_read (self):
        '''▶ notify that the connection is readable

    Send notification to the state machine that the
    connection is readable. Typically this is called after
    your main loop has detected that the file descriptor
    associated with this connection is readable.
'''
        return libnbdmod.aio_notify_read (self._o)

    def aio_notify_write (self):
        '''▶ notify that the connection is writable

    Send notification to the state machine that the
    connection is writable. Typically this is called after
    your main loop has detected that the file descriptor
    associated with this connection is writable.
'''
        return libnbdmod.aio_notify_write (self._o)

    def aio_is_created (self):
        '''▶ check if the connection has just been created

    Return true if this connection has just been created.
    This is the state before the handle has started
    connecting to a server. In this state the handle can
    start to be connected by calling functions such as
    "nbd.aio_connect".
'''
        return libnbdmod.aio_is_created (self._o)

    def aio_is_connecting (self):
        '''▶ check if the connection is connecting or handshaking

    Return true if this connection is connecting to the
    server or in the process of handshaking and negotiating
    options which happens before the handle becomes ready to
    issue commands (see "nbd.aio_is_ready").
'''
        return libnbdmod.aio_is_connecting (self._o)

    def aio_is_ready (self):
        '''▶ check if the connection is in the ready state

    Return true if this connection is connected to the NBD
    server, the handshake has completed, and the connection
    is idle or waiting for a reply. In this state the handle
    is ready to issue commands.
'''
        return libnbdmod.aio_is_ready (self._o)

    def aio_is_processing (self):
        '''▶ check if the connection is processing a command

    Return true if this connection is connected to the NBD
    server, the handshake has completed, and the connection
    is processing commands (either writing out a request or
    reading a reply).

    Note the ready state ("nbd.aio_is_ready") is not
    included. In the ready state commands may be *in flight*
    (the *server* is processing them), but libnbd is not
    processing them.
'''
        return libnbdmod.aio_is_processing (self._o)

    def aio_is_dead (self):
        '''▶ check if the connection is dead

    Return true if the connection has encountered a fatal
    error and is dead. In this state the handle may only be
    closed. There is no way to recover a handle from the
    dead state.
'''
        return libnbdmod.aio_is_dead (self._o)

    def aio_is_closed (self):
        '''▶ check if the connection is closed

    Return true if the connection has closed. There is no
    way to reconnect a closed connection. Instead you must
    close the whole handle.
'''
        return libnbdmod.aio_is_closed (self._o)

    def aio_command_completed (self, handle):
        '''▶ check if the command completed

    Return true if the command completed. If this function
    returns true then the command was successful and it has
    been retired. Return false if the command is still in
    flight. This can also fail with an error in case the
    command failed (in this case the command is also
    retired).

    The "handle" parameter is the positive unique 64 bit
    handle for the command, as returned by a call such as
    "nbd.aio_pread".
'''
        return libnbdmod.aio_command_completed (self._o, handle)

    def aio_peek_command_completed (self):
        '''▶ check if any command has completed

    Return the unique positive 64 bit handle of the first
    non-retired but completed command, 0 if no command is
    awaiting retirement, or -1 on error. Any handle returned
    by this function must still be passed to
    "aio_command_completed" to actually retire the command
    and learn whether the command was successful.
'''
        return libnbdmod.aio_peek_command_completed (self._o)

    def connection_state (self):
        '''▶ return a descriptive string for the state of the connection

    Returns a descriptive string for the state of the
    connection. This can be used for debugging or
    troubleshooting, but you should not rely on the state of
    connections since it may change in future versions.
'''
        return libnbdmod.connection_state (self._o)

    def get_package_name (self):
        '''▶ return the name of the library

    Returns the name of the library, always "libnbd" unless
    the library was modified with another name at compile
    time.
'''
        return libnbdmod.get_package_name (self._o)

    def get_version (self):
        '''▶ return a descriptive string for the state of the connection

    Return the version of libnbd. This is returned as a
    string in the form "major.minor.release" where each of
    major, minor and release is a small positive integer.
    For example "1.0.3".

    The major number is 0 for the early experimental
    versions of libnbd where we still had an unstable API,
    or 1 for the versions of libnbd with a long-term stable
    API and ABI.

    The minor number is even (0, 2, etc) for stable
    releases, and odd (1, 3, etc) for development versions.
    Note that new APIs added in a development version remain
    experimental and subject to change in that branch until
    they appear in a stable release.

    The release number is incremented for each release along
    a particular branch.
'''
        return libnbdmod.get_version (self._o)

    def supports_tls (self):
        '''▶ return true if libnbd was compiled with support for TLS

    Returns true if libnbd was compiled with gnutls which is
    required to support TLS encryption, or false if not. See
    "nbd.set_tls".
'''
        return libnbdmod.supports_tls (self._o)

    def supports_uri (self):
        '''▶ return true if libnbd was compiled with support for NBD URIs

    Returns true if libnbd was compiled with libxml2 which
    is required to support NBD URIs, or false if not. See
    "nbd.connect_uri" and "nbd.aio_connect_uri".
'''
        return libnbdmod.supports_uri (self._o)

package_name = NBD().get_package_name()
__version__ = NBD().get_version()

if __name__ == "__main__":
    import argparse
    import code
    import sys

    import nbd

    description = '''Network Block Device (NBD) shell'''
    parser = argparse.ArgumentParser (prog='nbdsh', description=description)
    parser.add_argument ('-V', '--version', action='version',
                         version=package_name + ' ' + __version__)
    args = parser.parse_args ()

    h = nbd.NBD ()
    sys.ps1 = "nbd> "

    banner = '''
Welcome to nbdsh, the shell for interacting with
Network Block Device (NBD) servers.

The ‘nbd’ module has already been imported and there
is an open NBD handle called ‘h’.

h.connect_tcp ("remote", "10809")   # Connect to a remote server.
h.get_size ()                       # Get size of the remote disk.
buf = h.pread (512, 0, 0)           # Read the first sector.
exit() or Ctrl-D                    # Quit the shell
help (nbd)                          # Display documentation
'''

    code.interact (banner = banner, local = locals(), exitmsg = '')
